package xyz.brassgoggledcoders.boilerplate.lib.common.tileentities;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.MovingObjectPosition;
import xyz.brassgoggledcoders.boilerplate.lib.BoilerplateLib;
import xyz.brassgoggledcoders.boilerplate.lib.client.IBlockOverlayText;
import xyz.brassgoggledcoders.boilerplate.lib.common.blocks.SideType;
import xyz.brassgoggledcoders.boilerplate.lib.common.config.ConfigEntry;
import xyz.brassgoggledcoders.boilerplate.lib.common.registries.ConfigRegistry;
import xyz.brassgoggledcoders.boilerplate.mod.Boilerplate;

import java.util.Arrays;

/**
 * @author SkySom
 */
public abstract class TileEntitySidedBase extends TileEntityBase implements IBlockOverlayText
{
	private SideType[] sideConfig;

	public TileEntitySidedBase()
	{
		super();
		sideConfig = new SideType[6];
		Arrays.fill(sideConfig, SideType.NONE);
	}

	public void toggleSide(int side)
	{
		int type = sideConfig[side].ordinal();
		type++;
		if (type >= SideType.values().length || type < 0)
		{
			type = 0;
		}
		this.sideConfig[side] = SideType.values()[type];
		field_145850_b.func_175641_c(this.func_174877_v(), this.func_145838_q(), 0, 0);
	}

	public void setSideConfig(int side, SideType sideType)
	{
		this.sideConfig[side] = sideType;
	}

	public SideType getSideValue(int side)
	{
		return this.sideConfig[side];
	}

	@Override
	public void readFromNBTCustom(NBTTagCompound nbtTagCompound)
	{
		int[] array = nbtTagCompound.func_74759_k("sideConfig");
		if (this.sideConfig == null)
		{
			this.sideConfig = new SideType[6];
		}
		if (array != null)
		{
			for (int i = 0; i < array.length; i++)
			{
				this.sideConfig[i] = SideType.values()[array[i]];
			}
		}
	}

	@Override
	public void writeToNBTCustom(NBTTagCompound nbtTagCompound)
	{
		int[] array = new int[6];
		for (int i = 0; i < this.sideConfig.length; i++)
		{
			array[i] = this.sideConfig[i].ordinal();
		}
		nbtTagCompound.func_74783_a("sideConfig", array);
	}

	@Override
	public boolean func_145842_c(int id, int arg)
	{
		if (id == 0)
		{
			this.field_145850_b.func_175689_h(this.func_174877_v());
			return true;
		}
		return false;
	}

	public void sendBlockUpdate()
	{
		if (!field_145850_b.field_72995_K)
		{
			this.field_145850_b.func_175689_h(this.func_174877_v());
		}
	}

	@Override
	public String[] getOverlayText(EntityPlayer player, MovingObjectPosition mop, boolean tool)
	{
		if (tool && ConfigRegistry.getBoolean("colorblind", false))
		{
			SideType facing = sideConfig[mop.field_178784_b.ordinal()];
			SideType opposite = sideConfig[mop.field_178784_b.func_176734_d().ordinal()];
			return new String[] {
					BoilerplateLib.getProxy().translate("blockSide.facing") + ": " +
							BoilerplateLib.getProxy().translate("sidetype." + facing.name().toLowerCase()),
					BoilerplateLib.getProxy().translate("blockSide.opposite") + ": "
							+ BoilerplateLib.getProxy().translate("sidetype." + opposite.name().toLowerCase()) };
		}
		return null;
	}
}
